@@ -16,10 +16,11 @@ require 'tmail/mailp'
 
 class HeaderField
 
-  def initialize( fname, fbody )
+  def initialize( fname, fbody, strict )
     @name = fname
     fbody.strip!
     @body = fbody
+    @strict = strict
 
     @parsed = false
     @parsing = false
@@ -30,7 +31,7 @@ class HeaderField
 
     alias hf_original_new new
 
-    def new( fname, fbody )
+    def new( fname, fbody, strict = false )
       if self == HeaderField then
         temp = fname.downcase
         if /\Ax-/ === temp then
@@ -39,9 +40,9 @@ class HeaderField
           cls = STR2CLASS[temp] || UnknownH
         end
 
-        cls.hf_original_new( fname, fbody )
+        cls.hf_original_new( fname, fbody, strict )
       else
-        hf_original_new( fname, fbody )
+        hf_original_new( fname, fbody, strict )
       end
     end
 
@@ -52,7 +53,7 @@ class HeaderField
       if r then
         module_eval %^
           def #{nm}
-            unless @parsed then
+            unless @parsed or @parsing then
               parse
             end
             @#{nm}
@@ -62,9 +63,8 @@ class HeaderField
       if w then
         module_eval %^
           def #{nm}=( arg )
-            unless @parsed and not @parsing then
+            unless @parsed or @parsing then
               parse
-              @parsed = true
             end
             @#{nm} = arg
           end
@@ -74,7 +74,7 @@ class HeaderField
 
     def parse_on_create
       module_eval %^
-        def initialize( fname, fbody )
+        def initialize( fname, fbody, strict )
           super
           parse
         end
@@ -120,7 +120,7 @@ end
 
 class StringH < HeaderField
 
-  def initialize( fn, bo )
+  def initialize( fname, fbody, strict )
     super
     @deced = nil
   end
@@ -149,15 +149,9 @@ end
 
 class StructH < HeaderField
 
-  def initialize( nm, bo )
-    super
-    @comments = []
-  end
-
   def comments
-    unless @parsed then
+    unless @parsed or @parsing then
       parse
-      @parsed = true
     end
     @comments
   end
@@ -173,16 +167,23 @@ class StructH < HeaderField
   private
 
   def init
-    @comments.clear
+    @comments = []
   end
 
   def parse
-    unless @parsing then
-      @parsing = true
-      init
+    @parsing = true
+    init
+    if @strict then
       Mailp.parse( self, @body )
-      @parsing = false
+    else
+      begin
+        Mailp.parse( self, @body )
+      rescue ParseError
+        ;
+      end
     end
+    @parsing = false
+    @parsed = true
   end
 
 end
@@ -192,8 +193,8 @@ end
 
 class UnknownH < StructH
 
-  def initialize( fname, body )
-    super( fname, body )
+  def initialize( fname, fbody, strict )
+    super
     @deced = nil
   end
 
@@ -647,9 +648,9 @@ end
 
 class MsgidH < StructH
 
-  def initialize( nm, str )
+  def initialize( fname, fbody, strict )
     super
-    @msgid = str.strip
+    @msgid = fbody.strip
   end
 
   property :msgid, String, true, true
